from __future__ import absolute_import, unicode_literals

"""
DESCRIPTION: This module is part of the pyElizaChatbotClient package released in the
             context of Re-WoChat: Workshop on Collecting and Generating Resources
             for Chatbots and Conversational Agents - Development and Evaluation
             (see further details at http://workshop.colips.org/re-wochat/)
             as part of the shared task (http://workshop.colips.org/re-wochat/shared.html).

USAGE: This program can be run from the command line by using:
             python createXMLFromJson.py -log pyEliza.log -user_name John -chatbot_name Eliza


OUTPUT: A log file called pyEliza.log, unless the user specifies another name during
        the initialization

VERSION: 0.1.0
DATE: Jan/28/2016
AUTHORS: Luis Fernando D'Haro, Rafael E. Banchs
EMAILS: luisdhe@i2r.a-star.edu.sg, rembanchs@i2r.a-star.edu.sg

"""

import os
import codecs
import sys
import traceback
import argparse

# External packages
try:
    from nltk import word_tokenize
except ImportError:
    try:
        import nltk
        nltk.download('punkt')
    except:
        raise ImportError('This program cannot run without NLTK and its tokenizer module.'
                          'use: pip install nltk')
    else:
        from nltk import word_tokenize

try:
    import petl as etl
except ImportError:
    raise ImportError('This program cannot run without petl module. Use pip to install it')

from xml.etree import ElementTree
from xml.etree.ElementTree import Element, SubElement
from xml.dom import minidom



def prettify(elem):
    """Return a pretty-printed XML string for the Element"""
    rough_string = ElementTree.tostring(elem, 'utf-8')
    reparsed = minidom.parseString(rough_string)
    return reparsed.toprettyxml(indent="  ")


def process_text(txt, chatbot_name=None, user_name=None):
    # Remove whitespaces
    txt = ' '.join(txt.split())
    if chatbot_name is not None or user_name is not None:
        words = word_tokenize(txt)
        if chatbot_name is not None:
            for pos in [i for i, x in enumerate(words) if x.lower() == chatbot_name.lower()]:
                words[pos] = 'SYSTEM_NAME'

        if user_name is not None:
            for pos in [i for i,x in enumerate(words) if x.lower() == user_name.lower()]:
                words[pos] = 'USER_NAME'

        txt = ' '.join(words)
    return txt


def fntCreateXMLFromLogs(chatbot_name='Eliza', user_name='John', logfile_name='pyEliza.log'):

    if logfile_name is None: logfile_name = 'pyEliza.log'
    if chatbot_name is None: chatbot_name = 'Eliza'
    if user_name is None: user_name = 'UNKNOWN'

    out_xml_file = os.path.splitext(os.path.abspath(logfile_name))[0]+".xml"
    print('Creating XML (%s) from Log file (%s)' %(out_xml_file, logfile_name))

    try:
        dialogues_xml = Element("dialogues")
        header_row = ['date', 'module', 'level', 'type_msg', 'msg']
        info_logs = etl.fromcsv(logfile_name, delimiter=str(u'|'))
        info_logs = etl.pushheader(info_logs, header_row)
        selected_info_logs = etl.select(info_logs,
                                        lambda rec: (str(rec.type_msg).strip() == 'MSG_USER' or
                                                    str(rec.type_msg).strip() == 'MSG_CHATBOT' or
                                                    str(rec.type_msg).strip() == 'MSG_WS_USER' or
                                                    str(rec.type_msg).strip() == 'MSG_WS_CHATBOT' or
                                                    str(rec.type_msg).strip() == 'MSG_WS_CHATBOT' or
                                                    str(rec.type_msg).strip() == 'USER_NAME' or
                                                    str(rec.type_msg).strip() == 'CHATBOT_NAME' or
                                                    str(rec.type_msg).strip() == 'INIT_CHAT' or
                                                    str(rec.type_msg).strip() == 'CLOSING')
                                                    and str(rec.level).strip() == 'INFO'
                                        )

        dialog_xml = None
        num_dialogs = 0
        turns_counter = 1
        bool_use_buffer = True
        buffer_turns = []  # Keep turns until the chatbot name and user name are set
        for line_log, turn in enumerate(selected_info_logs[1:]):  # Skip reading the header
            # print '... processing line ' + str(line_log+1)
            turn_dict = dict((header_row[y], x) for y, x in enumerate(turn))
            if 'INIT_CHAT' in turn_dict['type_msg']:
                buffer_turns = []
                turns_counter = 1
                num_dialogs += 1
                dialog_xml = SubElement(dialogues_xml, 'dialog', {'id': str(num_dialogs)})
                bool_use_buffer = True
                print ('Start processing dialog: %s' % num_dialogs)
            elif 'CHATBOT_NAME' in turn_dict['type_msg']:
                chatbot_name_xml = SubElement(dialog_xml, 'system_name')
                chatbot_name_xml.text = chatbot_name
            elif 'USER_NAME' in turn_dict['type_msg']:
                user_name_xml = SubElement(dialog_xml, 'user_name')
                user_name_xml.text = user_name
                if len(buffer_turns) > 0:  # Save the buffer and stop using it
                    for turn_xml in buffer_turns:
                        dialog_xml.append(turn_xml)
                    buffer_turns = []
                bool_use_buffer = False

            elif 'CLOSING' in turn_dict['type_msg']:
                if len(buffer_turns) > 0:
                    for turn_xml in buffer_turns:
                        dialog_xml.append(turn_xml)
                    buffer_turns = []
                print ('End processing dialog')
            else:
                turn_xml = Element('turn', {'id': str(turns_counter)})
                speaker_info = SubElement(turn_xml, 'speaker')
                if 'CHATBOT' in turn_dict['type_msg']:
                    speaker_info.text = 'SYSTEM'
                else:
                    speaker_info.text = 'USER'

                utterance_info = SubElement(turn_xml, 'utterance')
                if 'WS' in turn_dict['type_msg']:
                    jmsg = eval(turn_dict['msg'])
                    if 'USER' in turn_dict['type_msg']:
                        utterance_info.text = process_text(jmsg['payload'], chatbot_name, user_name)
                    else:
                        utterance_info.text = process_text(jmsg['payload'][0], chatbot_name, user_name)
                else:
                    utterance_info.text = process_text(turn_dict['msg'], chatbot_name, user_name)

                timestamp_info = SubElement(turn_xml, 'timestamp')
                timestamp_info.text = process_text(turn_dict['date'])
                if bool_use_buffer is True:
                    buffer_turns.append(turn_xml)
                else:
                    dialog_xml.append(turn_xml)
                turns_counter += 1

        with codecs.open(out_xml_file, 'w', 'utf-8') as f:
            f.write(prettify(dialogues_xml))

        print ('Successful conversion. Please submit the XML and Log files to the organizers')

    except Exception as e:
        print ('ERROR creating XML (%s) from Log file (%s)' %(out_xml_file, logfile_name))
        print ('Please submit the original Log files to the organizers')
        print("Unexpected error: %s" % sys.exc_info()[0])
        traceback.print_exc(file=sys.stdout)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Available options for generating '
                                                 'the XML file from a log file')
    parser.add_argument('-log', '--log', default='pyEliza.log', help='Specify the log '
                                                                     'file to convert. Default=pyEliza.log')
    parser.add_argument('-chatbot', '--chatbot', default='Eliza', help='Specify the name of the chatbot. Default=Eliza')
    parser.add_argument('-user_name', '--user_name', default='John', help='Specify the name of the user. Default=John')
    args = parser.parse_args()

    fntCreateXMLFromLogs(chatbot_name=args.chatbot, user_name=args.user_name,
                         logfile_name=args.log)


#!/usr/bin/env python
from __future__ import absolute_import, unicode_literals

"""
DESCRIPTION: This package is released in the context of Re-WoChat: Workshop on Collecting
             and Generating Resources for Chatbots and Conversational Agents - Development
             and Evaluation (see further details at http://workshop.colips.org/re-wochat/)
             as part of the shared task (http://workshop.colips.org/re-wochat/shared.html).

USAGE: This program can be run from the command line by using: python pyElizaChatbot.py
       The program can be imported from other python programs using:

       from pyelizachatbotclient import pyElizaChatbot
       client = pyElizaChatbot(name='Eliza', log_file='pyEliza.log')
       client.start()

OUTPUT: A log file called pyEliza.log, unless the user specifies another name during
        the initialization

VERSION: 0.1.4
DATE: Jan/29/2016
AUTHORS: Luis Fernando D'Haro, Rafael E. Banchs
EMAILS: luisdhe@i2r.a-star.edu.sg, rembanchs@i2r.a-star.edu.sg

"""

__version__ = "0.1.5"

# Standard Python Packages
import logging
import sys
import atexit
import random
import string
import time
from builtins import input

# External packages
try:
    from nltk import word_tokenize
except ImportError:
    try:
        import nltk
        nltk.download('punkt')
    except:
        raise ImportError('This program cannot run without NLTK and its tokenizer module.'
                          'use: pip install nltk')
    else:
        from nltk import word_tokenize


# Special programs/scripts
from .eliza import eliza
from .utils import createXMLfromJson


# Global variables
_name_logfile = None
_name_chatbot = None
_name_user = None

# Handling the process to create the XML when the system exits
def on_exit():
    # Function to create the XML at the end of the process
    createXMLfromJson.fntCreateXMLFromLogs(chatbot_name=_name_chatbot,
                                           user_name=_name_user,
                                           logfile_name=_name_logfile)
atexit.register(on_exit)


class pyElizaChatbot():

    # A dictionary of greetings to get the user's name
    dictNameGreeting = ['Nice to meet you __USR_NAME__ !', '__USR_NAME__ ... What a nice name.',
                        'I am not sure if we have already met __USR_NAME__, but anyway...',
                        'Great __USR_NAME__,']

    # A dictionary of follow up greetings for the chatbot
    dictStartingGreetings = ['What can I do for you?', 'What do you want to talk about?',
                             'How might I help you?', 'How are you doing today?',
                             'How is it going?', 'What''s up?']

    def __init__(self, chatbot_name='Eliza', log_file='pyEliza.log'):
        global _name_logfile
        global _name_chatbot

        self.chatbot_name = chatbot_name
        self.therapist = eliza.eliza()
        self.logger = self.init_logger(log_file)
        _name_logfile = log_file

        self.server_info = {
            'server_url': 'ws://localhost:8888/ws_chat',
            'host': 'localhost',
            'port': 8888,
        }

        self.client_info = {
            "id": 'Ee2oTw5qDMBtiMZ6',  # Identifier to access the server
            "max_users": 1,  # Integer. Indicates how many simultaneous users can handle the chatbot
                             # (WARNING: No implemented yet)
            "image": 'eliza-chatbot.gif',  # Image to show to the users
            "name": self.chatbot_name,  # Official name of the chatbot. Must be different
                              # from others available chatbots
            "description": 'The most famous psychoterapist',  # A short description of the chatbot
        }
        _name_chatbot = chatbot_name


    def init_logger(self, log_file='pyEliza.log',
                    ch_level=logging.DEBUG,
                    f_level=logging.DEBUG):
        global _name_logfile

        logger = logging.getLogger(__name__)
        logger.handlers = []
        logger.setLevel(logging.DEBUG)  # By default we want everything
        formatter = logging.Formatter('%(asctime)-15s | %(name)s | %(levelname)s | %(message)s')

        # File handler showing only info level
        f_handler = logging.FileHandler(log_file)
        f_handler.setLevel(f_level)
        f_handler.setFormatter(formatter)
        logger.addHandler(f_handler)

        # Console handler showing debug level
        ch_handler = logging.StreamHandler()
        ch_handler.setLevel(ch_level)
        ch_handler.setFormatter(formatter)
        logger.addHandler(ch_handler)

        _name_logfile = log_file

        return logger

    # Simplistic approach to get the name of the user
    def fntGetUserName(self, txt):
        remove_punctuation_map = dict((ord(char), None) for char in string.punctuation)
        thelist = ['hi', self.chatbot_name, 'hello', 'okay', 'ok',
                 'my', 'name', 'is', 'they', 'call', 'me', 'i', 'am', "'", 'it', "i'm"]
        words = word_tokenize(txt.lower())
        for pos in [i for i, x in enumerate(words) if x in thelist]:
            words[pos] = ''

        userName = ' '.join(words)
        userName = userName.translate(remove_punctuation_map)  # Remove punctuation
        userName = ' '.join(userName.split())
        return userName.capitalize()

    def findRandomAnswer(self, dictionary, name_replace=''):
        random.seed()
        random_num = random.randint(0, len(dictionary)-1)
        answer = dictionary[random_num]
        if len(name_replace.split()) > 0:
            answer = answer.replace('__USR_NAME__', name_replace)
        return answer

    def start(self):
        global _name_user
        print ('\n***********    WELCOME to pyEliza    ************')
        print ('\nPlease type the word: bye or press ctrl-c to exit\n')
        print('**************************************************\n')
        # Re-initialize the logger so the terminal is only showing the turns (i.e. no logs)
        self.init_logger(ch_level=logging.ERROR)

        # Procedure to introduce the chatbot and get the user's name (required when
        # using the chatbot as stand-alone app. When used as a websocket client
        # there will be previously a login process
        self.logger.info('INIT_CHAT | START')
        self.logger.info('CHATBOT_NAME | %s' % _name_chatbot.capitalize())
        start_time = time.time()
        init_salutation = self.chatbot_name + ': Hello, my name is ' + self.chatbot_name \
                     + '. Please tell me your name.'
        print (init_salutation)
        self.logger.info('MSG_CHATBOT | %s' % init_salutation)
        user_input = input('User: ')
        self.logger.info('MSG_USER | %s' % user_input)
        _name_user = self.fntGetUserName(user_input)
        self.logger.info('USER_NAME | %s' % _name_user)
        end_salutation = self.findRandomAnswer(self.dictNameGreeting, _name_user) \
                         + ' ' + self.findRandomAnswer(self.dictStartingGreetings)
        print(end_salutation)
        self.logger.info('MSG_CHATBOT | %s' % end_salutation)

        # Let's start the dialogue
        while True:
            try:
                user_input = input('User: ')
                self.logger.info('MSG_USER | %s' % user_input)
                chat_answer = self.therapist.respond(user_input)
                print(self.chatbot_name + ': ' + chat_answer)
                self.logger.info('MSG_CHATBOT | %s' % chat_answer)
                if user_input.lower() == 'bye':
                    self.logger.info("CLOSING | USER - duration: %s",
                                     time.time() - start_time)
                    exit()
            except KeyboardInterrupt:
                self.logger.info("CLOSING | KEYBOARD - duration: %s",
                                 time.time() - start_time)
                exit()
            except Exception as e:
                self.logger.error("ERROR | Unexpected error: %s" % sys.exc_info()[0])
                self.logger.error('Backtrace', exc_info=True)
                exit()

def main():
    client = pyElizaChatbot()
    client.start()


if __name__ == '__main__':
    main()




from setuptools import setup, find_packages

setup(
    name='pyElizaChatbotClient',
    version='0.1.5',
    author="Luis Fernando D'Haro",
    author_email='luisdhe@i2r.a-star.edu.sg',
    license='LICENSE.txt',
    packages=find_packages(exclude=['contrib', 'docs']),
    include_package_data = True,
    package_data={'test': 'test/pyElizaChatbotClient.py'},
    url='http://workshop.colips.org/re-wochat/',
    description='Basic stand-alone/websocket client for using in the Re-WoChat Shared task.',
    long_description=open('README.txt').read(),
    keywords='chatbots eliza re-wochat shared-task',
    install_requires=[
        "nltk >= 3.1",
        "petl >= 1.1.0",
        "future >= 0.15.2",
    ],
)

